@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2019 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -303,17 +303,43 @@ class Base(appier.Model):
         )
 
     @classmethod
-    def apply_views(cls, object, owner = None):
+    def apply_views(cls, object, views = None, owner = None):
+        """
+        Applies the complete set of views (object, class or callables)
+        to the current "query like" object so that it can be used to
+        filter the query context according to session.
+
+        This operation does not change the query object passed as parameter
+        as a copy is created for the mutation.
+
+        :type object: Dictionary
+        :param object: The query object to be changed according to the
+        set of views defined in session.
+        :type views: List
+        :param views: The list of views to be used in the apply views
+        operation, if not provided the current set of views in sessions
+        are going to be used instead.
+        :type owner: App
+        :param owner: The owner application to be used in the retrieval
+        of the session, if not provided the global static app is used.
+        :rtype: Dictionary
+        :return: The final mutated object ready to be used for view like
+        queries.
+        """
+
         # tries to retrieve the reference to the owner of the current
         # context or uses the global one otherwise (fallback)
         owner = owner or appier.get_app()
 
-        # verifies if there are any view defined under the current
-        # session if that's not the case returns immediately as there's
-        # nothing left to be filtered, otherwise retrieves the views to
-        # be used to filter the current object context
-        if not "views" in owner.session: return object
-        views = owner.session["views"]
+        # in case no views are explicitly provided via parameter, then
+        # the views must be retrieved from the current session
+        if views == None:
+            # verifies if there are any views defined under the current
+            # session if that's not the case returns immediately as there's
+            # nothing left to be filtered, otherwise retrieves the views to
+            # be used to filter the current object context
+            if not "views" in owner.session: return object
+            views = owner.session["views"]
 
         # creates a copy of the object so that it does not get modified
         # by the operation to be applied to it (avoids possible issues)
@@ -323,27 +349,112 @@ class Base(appier.Model):
         # session to be able to update the object accordingly, constraining
         # the context of resolution of that object (less results)
         for view in views:
-            if appier.legacy.is_str(view):
-                view_cls = owner.get_model(view)
-                view = view_cls.view_r
-            is_class = inspect.isclass(view)
-            if is_class:
-                view = view.view_r
-            is_callable = hasattr(view, "__call__")
-            if is_callable:
-                view = view(target = cls, owner = owner)
+            # resolves the current view, obtaining the proper map
+            # object, ready to be used in the update process
+            view = cls._resolve_view(view, resolve_name = "view_r")
+
+            # in case there's no view (invalid value) then continues
+            # the loop as no modification should be applied to object
             if not view: continue
+
+            # updates the (object) with the view map that has been
+            # resolved, effectively changing its behaviour
             object.update(view)
 
         # returns the final object to the caller method so that it can be
         # used to constrain contexts according to the current session views
         return object
 
+    @classmethod
+    def ensure_views(cls, object, ensure_set = True, views = None, owner = None):
+        # tries to retrieve the reference to the owner of the current
+        # context or uses the global one otherwise (fallback)
+        owner = owner or appier.get_app()
+
+        # in case no views are explicitly provided via parameter, then
+        # the views must be retrieved from the current session
+        if views == None:
+            # verifies if there are any views defined under the current
+            # session if that's not the case returns immediately as there's
+            # nothing left to be filtered, otherwise retrieves the views to
+            # be used to filter the current object context
+            if not "views" in owner.session: return object
+            views = owner.session["views"]
+
+        # iterates over the complete set of views defined under the current
+        # session to be able to update the object accordingly, validating
+        # the context of resolution of that object (less results)
+        for view in views:
+            # resolves the current view, obtaining the proper map
+            # object, ready to be used in the update process
+            view = cls._resolve_view(view, resolve_name = "view_l")
+
+            # in case there's no view (invalid value) then continues
+            # the loop as no validation needed for the object
+            if not view: continue
+
+            # iterates over the complete set of views elements to
+            # make sure the associated model attributes are defined
+            # accordingly raising exceptions otherwise
+            for name, values in appier.legacy.iteritems(view):
+                # in case the element is not required to be set
+                # at the object and it's not set the skips the
+                # current iteration loop no verification needed
+                if not ensure_set and not name in object:
+                    continue
+
+                appier.verify(
+                    name in object,
+                    message = "Attribute '%s' not set in object" % name,
+                    exception = appier.SecurityError
+                )
+
+                if not isinstance(values, (list, tuple)):
+                    values = [values]
+
+                appier.verify(
+                    object[name] in values,
+                    message = "Attribute '%s' not valid according to view" % name,
+                    exception = appier.SecurityError
+                )
+
     @classmethod
     @appier.operation(name = "Empty All", level = 2, devel = True)
     def op_empty_s(cls):
         cls.delete_c()
 
+    @classmethod
+    def _resolve_view(cls, view, resolve_name = "view_r", owner = None):
+        # tries to retrieve the reference to the owner of the current
+        # context or uses the global one otherwise (fallback)
+        owner = owner or appier.get_app()
+
+        # in case the value for the view is a string like
+        # value then it's considered to be the model from
+        # which the view resolver should be used to retrieve the
+        # concrete map based view
+        if appier.legacy.is_str(view):
+            view_cls = owner.get_model(view)
+            view = getattr(view_cls, resolve_name) if\
+                hasattr(view_cls, resolve_name) else None
+
+        # in case the view value is a class then the view is
+        # re-converted with the resolver method, to be called
+        is_class = inspect.isclass(view)
+        if is_class:
+            view = getattr(view, resolve_name) if\
+                hasattr(view, resolve_name) else None
+
+        # in case the view is a callable value then calls it
+        # to retrieve the concrete view (should be a map)
+        is_callable = hasattr(view, "__call__")
+        if is_callable:
+            view = view(target = cls, owner = owner)
+
+        # returns the "final" view value after the complete
+        # resolution process has been finished
+        return view
+
     @classmethod
     def _csv_import(
         cls,
@@ -532,6 +643,15 @@ class Base(appier.Model):
 
         self.destroy_index()
 
+    def save_v(self, *args, **kwargs):
+        is_new = kwargs.get("is_new", None)
+        before_callbacks = kwargs.get("before_callbacks", [])
+        if is_new == None: is_new = self.is_new()
+        ensure_callback = lambda instance, model: self.__class__.ensure_views(model)
+        before_callbacks.append(ensure_callback)
+        kwargs["before_callbacks"] = before_callbacks
+        appier.Model.save(self, *args, **kwargs)
+
     def previous(self, name = "id", raise_e = False, *args, **kwargs):
         kwargs[name] = {"$lt" : getattr(self, name)}
         kwargs["sort"] = ((name, -1),)
@@ -725,8 +845,8 @@ class Base(appier.Model):
 
     @property
     def created_d(self):
-        return datetime.datetime.fromtimestamp(self.created)
+        return datetime.datetime.utcfromtimestamp(self.created)
 
     @property
     def modified_d(self):
-        return datetime.datetime.fromtimestamp(self.modified)
+        return datetime.datetime.utcfromtimestamp(self.modified)
