@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2019 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -253,6 +253,7 @@ class AdminPart(
             (("POST",), "/admin/accounts", self.create_account),
             (("GET",), "/admin/accounts/me", self.me_account),
             (("GET",), "/admin/accounts/export.json", self.export_accounts_json, None, True),
+            (("GET",), "/admin/accounts/duplicates.json", self.duplicates_accounts_json, None, True),
             (("GET",), "/admin/accounts/<str:username>", self.show_account),
             (("GET",), "/admin/accounts/<str:username>/mail", self.mail_account),
             (("GET",), "/admin/accounts/<str:username>/avatar", self.avatar_account),
@@ -294,10 +295,13 @@ class AdminPart(
             (("GET",), "/admin/configs.json", self.show_configs, None, True),
             (("GET",), "/api/admin/ping", self.ping_api, None, True),
             (("GET", "POST"), "/api/admin/login", self.login_api, None, True),
+            (("GET", "POST"), "/api/admin/logout", self.logout_api, None, True),
             (("GET", "POST"), "/api/admin/oauth/access_token", self.oauth_access_token_api, None, True),
             (("GET", "POST"), "/api/admin/oauth/login", self.oauth_login_api, None, True),
             (("POST",), "/api/admin/database/reset", self.database_reset_api, None, True),
             (("GET",), "/api/admin/accounts/me", self.me_account_api, None, True),
+            (("GET",), "/api/admin/sessions/me", self.show_session_me_api, None, True),
+            (("GET",), "/api/admin/sessions/<str:sid>", self.show_session_api, None, True),
             (("GET",), "/api/admin/models/<str:model>", self.show_model_api, None, True),
             (("GET",), "/api/admin/models/<str:model>/<str:_id>", self.show_entity_api, None, True)
         ]
@@ -639,7 +643,7 @@ class AdminPart(
             account = account
         )
 
-    @appier.ensure(context = "admin")
+    @appier.ensure(token = "admin.accounts", context = "admin")
     def export_accounts_json(self):
         account_c = self._get_cls(self.account_c)
         object = appier.get_object(
@@ -654,6 +658,29 @@ class AdminPart(
             **object
         )
 
+    @appier.ensure(token = "admin.accounts", context = "admin")
+    def duplicates_accounts_json(self):
+        account_c = self._get_cls(self.account_c)
+        object = appier.get_object(
+            alias = True,
+            find = True,
+            limit = 0
+        )
+        accounts = account_c.find(
+            account_c,
+            rules = False,
+            **object
+        )
+        keys_m = dict()
+        for account in accounts:
+            sequence = keys_m.get(account.key, [])
+            sequence.append(account.username)
+            keys_m[account.key] = sequence
+        for key, values in appier.legacy.items(keys_m):
+            if len(values) > 1: continue
+            del keys_m[key]
+        return dict(result = keys_m)
+
     @appier.ensure(token = "admin.accounts", context = "admin")
     def show_account(self, username):
         account_c = self._get_cls(self.account_c)
@@ -862,7 +889,9 @@ class AdminPart(
                 **{
                     "$or": [
                         dict(redirect_uri = redirect_uri),
-                        dict(redirect_uri = "http://*")
+                        dict(redirect_uri = "http://*"),
+                        dict(redirect_uri = "https://*"),
+                        dict(redirect_uri = "file://*")
                     ]
                 }
             )
@@ -1222,6 +1251,7 @@ class AdminPart(
             session_s = self.request.session_c.get_s(sid)
         )
 
+    @appier.ensure(token = "admin.status", context = "admin")
     def delete_session_me(self):
         sid = self.session.sid
         self.request.session_c.expire(sid)
@@ -1717,7 +1747,7 @@ class AdminPart(
     def create_entity(self, model):
         model = self.get_model(model)
         entity = model.new(safe = False)
-        try: entity.save()
+        try: entity.save_v()
         except appier.ValidationError as error:
             return self.template(
                 "entities/new.html.tpl",
@@ -1793,7 +1823,7 @@ class AdminPart(
             _id = self.get_adapter().object_id(_id)
         )
         entity.apply(safe_a = False)
-        try: entity.save()
+        try: entity.save_v()
         except appier.ValidationError as error:
             return self.template(
                 "entities/edit.html.tpl",
@@ -2213,12 +2243,34 @@ class AdminPart(
             tokens = account.tokens()
         )
 
+    def logout_api(self):
+        # verifies the existence of the various account related session
+        # attributes and in case they exist removes them from session as
+        # the user is currently logging out from session
+        self.account_c._unset_account()
+
+        # returns a valid result indicating that the operation has been
+        # executed with proper success
+        return True
+
     @appier.ensure(context = "admin")
     def me_account_api(self):
         account_c = self._get_cls(self.account_c)
         account = account_c.from_session(map = True)
         return account
 
+    @appier.ensure(context = "admin")
+    def show_session_me_api(self):
+        sid = self.session.sid
+        session_s = self.request.session_c.get_s(sid)
+        return dict(session_s.items())
+
+    @appier.ensure(context = "admin")
+    def show_session_api(self, sid):
+        sid = str(sid)
+        session_s = self.request.session_c.get_s(sid)
+        return dict(session_s.items())
+
     @appier.ensure(context = "admin")
     def show_model_api(self, model):
         return self.show_model_json(model)
@@ -2300,7 +2352,9 @@ class AdminPart(
         versions = []
         if hasattr(module, "VERSION"):
             versions.append(("PIL", module.VERSION))
-        if hasattr(module, "PILLOW_VERSION"):
+        if hasattr(module, "__version__"):
+            versions.append(("Pillow", module.__version__))
+        elif hasattr(module, "PILLOW_VERSION"):
             versions.append(("Pillow", module.PILLOW_VERSION))
         return versions
 
